import jwtDecode from 'jwt-decode';
import { machine, enhance, XModel, XMsg, XCmd, state } from 'tesm/dist/ts'
import { СloudPaymentsWebAPI } from '../api';
import { SubscriptionCurrentType, TokenType } from "../types";

export const _state = <T extends {}>() => (m: T) => m

type State = {
	sidebarOpen: boolean
	email: string
	passcode: string
	subscriptionCurrent: SubscriptionCurrentType
	userId: number
}

const DEFAULT_STATE: State = {
	sidebarOpen: false,
	email: '',
	passcode: '',
	userId: 0,
	subscriptionCurrent: {
		"subscriptionId": 0,
		"duration": "",
        "endDate": "",
        "price": 0,
        "currency": 0,
        "isActive": true,
        "isAutoRenewStatus": true,
        "subscriptionKind": 3,
        "paymentMethod": 5,
        "name": "",
        "isIntroPeriod": false,
        "isTrialPeriod": false
	}
}

export type ScreenType = keyof typeof ScreensState

const ScreensState = state({
	not_auth: _state<State>(),
	auth: _state<State>(),
	start: _state<State>(),
	subscription: _state<State>(),
	about: _state<State>(),
	instruction: _state<State>(),
	cancelation: _state<State>(),
})

const m = machine(
	//states
	ScreensState, {
	// messages
	set_screen: (screen: ScreenType) => ({ screen }),
	request_auth: (token: string) => ({ token }),
	success_auth: () => ({}),
	failed_auth: () => ({}),
	set_sidebar_open: () => ({}),
	set_email: (email: string) => ({ email }),
	set_userid: (userid: number) => ({ userid }),
	set_passcode: (passcode: string) => ({ passcode }),
	set_subscription_current: (subscriptionCurrent: SubscriptionCurrentType) => ({ subscriptionCurrent }),
	update_subscription_current: (subscriptionCurrent: Partial<SubscriptionCurrentType>) => ({ subscriptionCurrent }),
	get_subscription_current: () => ({}),
	authorize: (data: { email: string, passcode: string }) => ({ data }),
	cancel_subscription: (subscriptionId: number) => ({ subscriptionId })
}, {
	// commands
	authorize: (data: { email: string, passcode: string }) => ({ data }),
	parse_token: (token) => ({ token }),
	get_subscription_current: () => ({}),
	cancel_subscription: (subscriptionId: number) => ({ subscriptionId })
})

export type Model = XModel<typeof m>
export type Msg = XMsg<typeof m>
export type Cmd = XCmd<typeof m>

export const StateMachine = enhance(
	m,
	"StateMachine",
	() => [m.states.not_auth(DEFAULT_STATE)],
	{
		not_auth: {
			request_auth: (msg, model) => [
				m.states.not_auth(model),
				m.cmds.parse_token(msg.token)
			],
			success_auth: (msg, model) => [
				m.states.start(model),
			],
			failed_auth: (msg, model) => [
				m.states.auth(model),
			],
		},
		auth: {
			authorize: (msg, model) => [
				m.states.auth({
					...model,
					email: msg.data.email,
					passcode: msg.data.passcode
				}),
				m.cmds.authorize(msg.data)
			],
			success_auth: (msg, model) => [
				m.states.start(model),
			],
			failed_auth: (msg, model) => [
				m.states.auth(model),
			],
		},
		start: {

		}
	}, {
	get_subscription_current: (msg, model) => [model, m.cmds.get_subscription_current()],
	cancel_subscription: (msg, model) => [model, m.cmds.cancel_subscription(msg.subscriptionId)],
	set_subscription_current: (msg, model) => [{ ...model, subscriptionCurrent: msg.subscriptionCurrent }],
	update_subscription_current: (msg, model) => [{ ...model, subscriptionCurrent: { ...model.subscriptionCurrent, ...msg.subscriptionCurrent } }],
	set_email: (msg, model) => [{ ...model, email: msg.email }],
	set_passcode: (msg, model) => [{ ...model, passcode: msg.passcode }],
	set_sidebar_open: (msg, model) => [{ ...model, sidebarOpen: !model.sidebarOpen }],
	set_screen: (msg, model) =>
	{
		switch (msg.screen)
		{
			case 'not_auth':
				return [m.states.not_auth(model)]
			case 'auth':
				return [m.states.auth(model)]
			case 'about':
				return [m.states.about(model)]
			case 'start':
				return [m.states.start(model)]
			case 'cancelation':
				return [m.states.cancelation(model)]
			case 'instruction':
				return [m.states.instruction(model)]
			case 'subscription':
				return [m.states.subscription(model)]
			default:
				return [m.states.auth(model)]
		}
	},
})

export const effectHandler = async (cmd: Cmd, send: (msg: Msg) => void) =>
{
	switch (cmd.type)
	{
		case 'cancel_subscription': {
			try
			{
				await СloudPaymentsWebAPI.cancelSubscription(cmd.subscriptionId)
				send(m.msgs.update_subscription_current({ subscriptionId: null }))
				send(m.msgs.set_screen('subscription'))
				return

			} catch (error)
			{
				return
			}
		}
		case 'get_subscription_current': {
			const sub_current = await СloudPaymentsWebAPI.getCurrentSubscription()

			send(m.msgs.set_subscription_current(sub_current))
			return
		}

		case 'authorize': {
			try
			{
				const auth_data = await СloudPaymentsWebAPI.auth(cmd.data)
				await СloudPaymentsWebAPI.setToken(auth_data.access_token)
				localStorage.setItem('token', auth_data.access_token)
				send(m.msgs.get_subscription_current())

				const tokenParsed: TokenType = jwtDecode(auth_data.access_token)

				console.log(tokenParsed)
				send(m.msgs.set_email(tokenParsed.email))
				switch (tokenParsed.client_id)
				{
					case 'Zebrainy.Skazbuka.Api.Client.Hybrid':
						send(m.msgs.set_screen('start'))
						break;
					case 'Zebrainy.Skazbuka.Api.Client.Web':
						send(m.msgs.set_screen('instruction'))
						break;
					default:
						send(m.msgs.set_screen('auth'))
						break;
				}
			} catch (error)
			{
				send(m.msgs.set_screen('auth'))
			}
			return
		}

		case 'parse_token': {
			try
			{
				const token = cmd.token
				await СloudPaymentsWebAPI.setToken(token)

				const data = await СloudPaymentsWebAPI.getPasscode()

				if (token)
				{
					const tokenParsed: TokenType = jwtDecode(token)
					send(m.msgs.set_email(tokenParsed.email))
					send(m.msgs.set_userid(tokenParsed.sub))
					switch (tokenParsed.client_id)
					{
						case 'Zebrainy.Skazbuka.Api.Client.Hybrid':
							send(m.msgs.set_screen('start'))
							break;
						case 'Zebrainy.Skazbuka.Api.Client.Web':
							send(m.msgs.set_screen('instruction'))
							break;
						default:
							break;
					}
				}

				send(m.msgs.set_passcode(data.passcode))
				send(m.msgs.get_subscription_current())
			} catch (error)
			{
				send(m.msgs.set_screen('auth'))
			}

			return
		}

		default:
			return
	}
}

export namespace Loading
{
	export type StateModel = Model
	export type StateMsg = ReturnType<typeof StateMachine.msgCreator>
	export type StateCmd = Cmd
}