import {IDatabase, IMain} from 'pg-promise';
import {IResult} from 'pg-promise/typescript/pg-subset';
import {DbTransaction, Transaction} from '../models';
import {transactions as sql} from '../sql';

/*
 This repository mixes hard-coded and dynamic SQL, just to show how to use both.
*/

export class TransactionsRepository {

    /**
     * @param db
     * Automated database connection context/interface.
     *
     * If you ever need to access other repositories from this one,
     * you will have to replace type 'IDatabase<any>' with 'any'.
     *
     * @param pgp
     * Library's root, if ever needed, like to access 'helpers'
     * or other namespaces available from the root.
     */
    constructor(private db: IDatabase<any>, private pgp: IMain) {
        /*
          If your repository needs to use helpers like ColumnSet,
          you should create it conditionally, inside the constructor,
          i.e. only once, as a singleton.
        */
    }

    async add(values: Transaction): Promise<DbTransaction> {
        var current_date = new Date()
        return this.db.one(sql.add,
            {
                created_date: current_date,
                created_by: 1,
                modified_date: current_date,
                modified_by: 1,
                ...values
            });
    }

    async findByUtid(utid: string): Promise<DbTransaction | null> {
        return this.db.oneOrNone('SELECT * FROM core.transaction WHERE utid = $1', utid);
    }

    async findByInvoiceId(invoiceId: string): Promise<DbTransaction | null> {
        return this.db.oneOrNone(sql.findByInvoiceId, {
            invoiceId: invoiceId,
        });
    }

    async findByUserId(userId: number): Promise<DbTransaction[] | null> {
        return this.db.oneOrNone(sql.findByUserId, {
            userId: userId,
        });
    }

    async processUserPayment(): Promise<void> {
        return this.db.tx(t => {
            // creating a sequence of transaction queries:
            const q1 = t.none('UPDATE users SET active = $1 WHERE id = $2', [true, 123]);
            const q2 = t.one('INSERT INTO audit(entity, id) VALUES($1, $2) RETURNING id', ['users', 123]);

            // returning a promise that determines a successful transaction:
            return t.batch([q1, q2]); // all of the queries are to be resolved;
        })
        .then(data => {
            // success, COMMIT was executed
        })
        .catch(error => {
            // failure, ROLLBACK was executed
        });
    }

}
